SystemOrganization addCategory: #'AntSimulation-Test'!
SystemOrganization addCategory: #'AntSimulation-Graphics'!
SystemOrganization addCategory: #AntSimulation!

TestCase subclass: #EnvironmentTest
	instanceVariableNames: 'env entity'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AntSimulation-Test'!

!EnvironmentTest methodsFor: 'running' stamp: 'VanceThornton 7/7/2012 00:12'!
setUp
	|cell|
	env := Environment new initWithRows: 10 columns: 10.
	
	self assert: env rowCount equals: 10.
	self assert: env columnCount equals: 10.
	self assert: env entities size equals: 0.

	entity := Entity new.
	cell := env cellAtRow: 5 column: 5.
	env add: entity to: cell.

! !

!EnvironmentTest methodsFor: 'testing' stamp: 'VanceThornton 7/7/2012 00:14'!
testAddEntity
	|newEntity cell entityStatus|

	newEntity := Entity new.	
	cell := env cellAtRow: 3 column: 2.
	env add: newEntity to: cell.
	
	"The environment contains one existing entity and the new entity"
	self assert: env entities size equals: 2.

	entityStatus := env statusForEntity: newEntity.
	self assert: entityStatus health equals: 1.0 .
	
	self assert: (entityStatus cell) == cell.
	self assert: (entityStatus cell) row equals: 3.
	self assert: (entityStatus cell) column equals: 2.! !

!EnvironmentTest methodsFor: 'testing' stamp: 'VanceThornton 7/7/2012 00:14'!
testCellDirectionOfCell
	|cell otherCell|

	cell := env cellAtRow: 5 column: 7.
	
	otherCell := env cell: #NORTH ofCell: cell.
	self assert: (otherCell) row equals: 4.
	self assert: (otherCell column) equals: 7.

	otherCell := env cell: #NORTH_EAST ofCell: cell.
	self assert: (otherCell) row equals: 4.
	self assert: (otherCell column) equals: 8.

	otherCell := env cell: #EAST ofCell: cell.
	self assert: (otherCell) row equals: 5.
	self assert: (otherCell column) equals: 8.

	otherCell := env cell: #SOUTH_EAST ofCell: cell.
	self assert: (otherCell) row equals: 6.
	self assert: (otherCell column) equals: 8.
	
	otherCell := env cell: #SOUTH ofCell: cell.
	self assert: (otherCell) row equals: 6.
	self assert: (otherCell column) equals: 7.

	otherCell := env cell: #SOUTH_WEST ofCell: cell.
	self assert: (otherCell) row equals: 6.
	self assert: (otherCell column) equals: 6.

	otherCell := env cell: #WEST ofCell: cell.
	self assert: (otherCell) row equals: 5.
	self assert: (otherCell column) equals: 6.

	otherCell := env cell: #NORTH_WEST ofCell: cell.
	self assert: (otherCell) row equals: 4.
	self assert: (otherCell column) equals: 6.

	"Test out of bounds cells"
	
	cell := env cellAtRow: 1 column: 7.
	self assert: (env cell: #NORTH ofCell: cell) == nil.
	self assert: (env cell: #NORTH_EAST ofCell: cell) == nil.
	self assert: (env cell: #NORTH_WEST ofCell: cell) == nil.
	
	cell := env cellAtRow: (env rowCount) column: 7.
	self assert: (env cell: #SOUTH ofCell: cell) == nil.
	self assert: (env cell: #SOUTH_EAST ofCell: cell) == nil.
	self assert: (env cell: #SOUTH_WEST ofCell: cell) == nil.

	cell := env cellAtRow: 7 column: 1.
	self assert: (env cell: #WEST ofCell: cell) == nil.
	self assert: (env cell: #NORTH_WEST ofCell: cell) == nil.
	self assert: (env cell: #SOUTH_WEST ofCell: cell) == nil.

	cell := env cellAtRow: 7 column: (env columnCount).
	self assert: (env cell: #EAST ofCell: cell) == nil.
	self assert: (env cell: #NORTH_EAST ofCell: cell) == nil.
	self assert: (env cell: #SOUTH_EAST ofCell: cell) == nil.

	"Test invalid direction"
	self should: [env cell: #UP ofCell: cell] raise: Error.
	
	"Test nil cell"
	self should: [env cell: #SOUTH_EAST ofCell: nil] raise: Error.! !

!EnvironmentTest methodsFor: 'testing' stamp: 'VanceThornton 7/7/2012 00:11'!
testMoveEntity
	|toCell entityStatus|

	toCell := env cellAtRow: 3 column: 3.
	env move: entity to: toCell.
	
	self assert: env entities size equals: 1.

	entityStatus := env statusForEntity: entity.
	
	self assert: (entityStatus cell) == toCell.
	self assert: (entityStatus cell) row equals: 3.
	self assert: (entityStatus cell) column equals: 3.! !

EnvironmentTest subclass: #MoveMotorResponseTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AntSimulation-Test'!

!MoveMotorResponseTest methodsFor: 'testing' stamp: 'VanceThornton 7/7/2012 00:18'!
testMoveBackward
	|entityStatus cell response|

	entityStatus := env statusForEntity: entity.
	entityStatus direction: #EAST.
	cell := entityStatus cell.
	
	response := MoveMotorResponse responseForEntity: entity moveDirection: #BACKWARD.
	
	response performInEnvironment: env.
	
	self assert: (cell entities size) equals: 0.
	
	cell := entityStatus cell.
	self assert: (cell entities size) equals: 1.	
	self assert: (cell row) equals: 5.
	self assert: (cell column) equals: 4.

	"Test moving backward at the boundary of the environment"	
	cell := env cellAtRow: 5 column: 1.
	env move: entity to: cell.
	response performInEnvironment: env.

	"The entity should remain in the original cell"
	cell := entityStatus cell.
	self assert: (cell row) equals: 5.
	self assert: (cell column) equals: 1.! !

!MoveMotorResponseTest methodsFor: 'testing' stamp: 'VanceThornton 7/7/2012 00:19'!
testMoveForward
	|entityStatus cell response|
	
	entityStatus := env statusForEntity: entity.
	entityStatus direction: #EAST.
	cell := entityStatus cell.

	response := MoveMotorResponse responseForEntity: entity moveDirection: #FORWARD.
	
	response performInEnvironment: env.
	
	self assert: (cell entities size) equals: 0.
	
	cell := entityStatus cell.
	self assert: (cell entities size) equals: 1.	
	self assert: (cell row) equals: 5.
	self assert: (cell column) equals: 6.

	"Test moving forward at the boundary of the environment"	
	cell := env cellAtRow: 5 column: (env columnCount).
	env move: entity to: cell.
	response performInEnvironment: env.

	"The entity should remain in the original cell"
	cell := entityStatus cell.
	self assert: (cell row) equals: 5.
	self assert: (cell column) equals: (env columnCount).! !

EnvironmentTest subclass: #TurnMotorResponseTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AntSimulation-Test'!

!TurnMotorResponseTest methodsFor: 'testing' stamp: 'VanceThornton 7/7/2012 00:27'!
testTurnLeft
	|entityStatus cell response|

	entityStatus := env statusForEntity: entity.
	entityStatus direction: #EAST.
	cell := entityStatus cell.
	
	response := TurnMotorResponse responseForEntity: entity turnDirection: #LEFT.	
	response performInEnvironment: env.

	self assert: entityStatus direction equals: #NORTH_EAST.
	self assert: entityStatus cell = cell.	
	
	"Turning left for the full circle (8 turns) should return the entity to the original direction"
	entityStatus direction: #NORTH.
	(1 to: 8) do: [:count|response performInEnvironment: env].
	self assert: entityStatus direction equals: #NORTH.
	
	
! !

!TurnMotorResponseTest methodsFor: 'testing' stamp: 'VanceThornton 7/7/2012 00:28'!
testTurnRight
	|entityStatus cell response|

	entityStatus := env statusForEntity: entity.
	entityStatus direction: #EAST.
	cell := entityStatus cell.
	
	response := TurnMotorResponse responseForEntity: entity turnDirection: #RIGHT.
	response performInEnvironment: env.

	self assert: entityStatus direction equals: #SOUTH_EAST.
	self assert: entityStatus cell = cell.	
	
	"Turning right for the full circle (8 turns) should return the entity to the original direction"
	entityStatus direction: #NORTH.
	(1 to: 8) do: [:count|response performInEnvironment: env].
	self assert: entityStatus direction equals: #NORTH.
	
	
! !

TestCase subclass: #SurfaceEnvironmentTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AntSimulation-Test'!

!SurfaceEnvironmentTest methodsFor: 'testing' stamp: 'VanceThornton 7/4/2012 00:27'!
testGenerateRandom
	|env|
	env := SurfaceEnvironment generateRandomWithRows: 25 columns: 25 antCount: 10.
	
	self assert: (env entities) size equals: 10.
	
	(env entities) do: [:entity |
		|entityStatus cell|
		entityStatus := env statusForEntity: entity.
		self assert: entityStatus ~= nil.
		
		cell := entityStatus cell.
		self assert: ((cell entities) includes: entity).		
		self assert: (env cellAtRow: (cell row) column: (cell column)) == cell.
	].! !

SimpleButtonMorph subclass: #AntSimulationCell
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AntSimulation-Graphics'!

!AntSimulationCell commentStamp: 'agallan2 7/4/2012 20:59' prior: 0!
I represent a single discrete location in the AntSimulation. I am a button, as clicking me returns information about the simulation object I represent.!

!AntSimulationCell methodsFor: 'accessing' stamp: 'agallan2 7/5/2012 08:25'!
defaultLabel
	^ ' '.! !

!AntSimulationCell methodsFor: 'initialization' stamp: 'agallan2 7/5/2012 08:52'!
initialize
	super initialize.
	self borderWidth: 2.
	bounds := 0@0 corner: 16@16.! !

BorderedMorph subclass: #GraphicalEnvironment
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AntSimulation-Graphics'!

!GraphicalEnvironment methodsFor: 'initialization' stamp: 'agallan2 7/5/2012 08:26'!
initialize
	super initialize.! !

GraphicalEnvironment subclass: #SurfaceEnvironmentView
	instanceVariableNames: 'cells'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AntSimulation-Graphics'!

!SurfaceEnvironmentView methodsFor: 'accessing' stamp: 'agallan2 7/5/2012 08:38'!
cellsPerSide
	^ 30.! !

!SurfaceEnvironmentView methodsFor: 'initialization' stamp: 'agallan2 7/5/2012 09:00'!
initialize
	| sampleCell width height n |
	super initialize.
	n := self cellsPerSide.
	sampleCell := AntSimulationCell new.
	width := sampleCell width.
	height := sampleCell height.
	self bounds: (5@5 extent: ((width*n) @(height*n)) + (2 * self borderWidth)).
	cells := Matrix new: n tabulate: [ :i :j | self newCellAt: i at: j ].! !

!SurfaceEnvironmentView methodsFor: 'initialization' stamp: 'agallan2 7/5/2012 08:53'!
newCellAt: i at: j
	"Creates a new AntSimulationCell at the given i,j coordinates."
	| c origin |
	c := AntSimulationCell new.
	origin := self innerBounds origin.
	self addMorph: c.
	c position: ((i - 1) * c width) @ ((j - 1) * c height) + origin.
! !

Object subclass: #AntSensoryInput
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AntSimulation'!

Object subclass: #AntSimulation
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AntSimulation'!

AntSimulation class
	instanceVariableNames: 'random'!

AntSimulation class
	instanceVariableNames: 'random'!

!AntSimulation class methodsFor: 'as yet unclassified' stamp: 'VanceThornton 7/4/2012 00:09'!
random
	(random == nil) ifTrue: [random := Random seed: 1].
	^random! !

Object subclass: #Cell
	instanceVariableNames: 'row column entities'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AntSimulation'!

!Cell methodsFor: 'as yet unclassified' stamp: 'VanceThornton 7/3/2012 23:03'!
add: entity
	entities add: entity! !

!Cell methodsFor: 'accessing' stamp: 'VanceThornton 7/3/2012 23:38'!
column
	^column! !

!Cell methodsFor: 'accessing' stamp: 'VanceThornton 7/4/2012 00:25'!
entities
	^entities! !

!Cell methodsFor: 'initialize-release' stamp: 'VanceThornton 7/4/2012 00:24'!
initWithRow: newRow column: newColumn
 	row := newRow.
	column := newColumn.
	entities := Set new.! !

!Cell methodsFor: 'as yet unclassified' stamp: 'VanceThornton 7/6/2012 00:49'!
remove: entity
	entities remove: entity! !

!Cell methodsFor: 'accessing' stamp: 'VanceThornton 7/3/2012 23:38'!
row
	^row! !

Object subclass: #Entity
	instanceVariableNames: 'id'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AntSimulation'!

Entity subclass: #ActiveEntity
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AntSimulation'!

ActiveEntity subclass: #Ant
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AntSimulation'!

!Ant class methodsFor: 'as yet unclassified' stamp: 'VanceThornton 7/7/2012 00:42'!
sensoryInputForEntity: entity inEnvironment: env
	|status|
	status := env statusForEntity: entity.
	^AntSensoryInput new.! !

!Ant methodsFor: 'as yet unclassified' stamp: 'VanceThornton 7/6/2012 00:40'!
responseTo: sensoryInput
	^NoMotorResponse new! !

!Entity class methodsFor: 'as yet unclassified' stamp: 'VanceThornton 7/6/2012 00:38'!
sensoryInputForEntity: entity inEnvironment: env
	self subclassResponsibility! !

!Entity methodsFor: 'comparing' stamp: 'VanceThornton 7/3/2012 18:58'!
= anEntity
	self == anEntity ifTrue: [^true].
	^(id = anEntity id)
! !

!Entity methodsFor: 'accessing' stamp: 'VanceThornton 7/3/2012 18:58'!
id
	^id! !

!Entity methodsFor: 'initialize-release' stamp: 'VanceThornton 7/4/2012 00:13'!
initialize
	id := UUID new.! !

!Entity methodsFor: 'as yet unclassified' stamp: 'VanceThornton 7/6/2012 00:32'!
responseTo: sensoryInput
	self subclassResponsibility! !

Entity subclass: #InactiveEntity
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AntSimulation'!

InactiveEntity subclass: #Food
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AntSimulation'!

InactiveEntity subclass: #Nest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AntSimulation'!

InactiveEntity subclass: #Pheromone
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AntSimulation'!

InactiveEntity subclass: #Rock
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AntSimulation'!

Object subclass: #EntityStatus
	instanceVariableNames: 'cell health direction'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AntSimulation'!

!EntityStatus methodsFor: 'accessing' stamp: 'VanceThornton 7/3/2012 23:36'!
cell
	^cell! !

!EntityStatus methodsFor: 'accessing' stamp: 'VanceThornton 7/6/2012 00:50'!
cell: newCell
	cell := newCell! !

!EntityStatus methodsFor: 'accessing' stamp: 'VanceThornton 7/6/2012 00:44'!
direction
	^direction! !

!EntityStatus methodsFor: 'accessing' stamp: 'VanceThornton 7/6/2012 22:54'!
direction: newDirection
	direction := newDirection! !

!EntityStatus methodsFor: 'accessing' stamp: 'VanceThornton 7/3/2012 23:35'!
health
	^health! !

!EntityStatus methodsFor: 'initialize-release' stamp: 'VanceThornton 7/6/2012 00:45'!
initWithCell: aCell
	cell := aCell.
	health := 1.0.
	direction := #north.
	! !

Object subclass: #Environment
	instanceVariableNames: 'cellMatrix entityStatusMap'
	classVariableNames: 'leftTurnDictionary reverseDirectionDictionary rightTurnDictionary'
	poolDictionaries: ''
	category: 'AntSimulation'!

!Environment class methodsFor: 'direction' stamp: 'VanceThornton 7/7/2012 00:33'!
directionAfterTurning: direction from: currentDirection
	direction == #LEFT ifTrue: [^Environment directionAfterTurningLeftFrom: currentDirection].
	direction == #RIGHT ifTrue: [^Environment directionAfterTurningRightFrom: currentDirection].
	^nil.! !

!Environment class methodsFor: 'direction' stamp: 'VanceThornton 7/7/2012 00:35'!
directionAfterTurningLeftFrom: currentDirection
	^leftTurnDictionary at: currentDirection.! !

!Environment class methodsFor: 'direction' stamp: 'VanceThornton 7/7/2012 00:35'!
directionAfterTurningRightFrom: currentDirection
	^rightTurnDictionary at: currentDirection.! !

!Environment class methodsFor: 'initialize-release' stamp: 'VanceThornton 7/7/2012 00:38'!
initialize
	leftTurnDictionary := Dictionary new.
	leftTurnDictionary add: (Association key: #NORTH value: #NORTH_WEST).
	leftTurnDictionary add: (Association key: #NORTH_WEST value: #WEST).
	leftTurnDictionary add: (Association key: #WEST value: #SOUTH_WEST).
	leftTurnDictionary add: (Association key: #SOUTH_WEST value: #SOUTH).
	leftTurnDictionary add: (Association key: #SOUTH value: #SOUTH_EAST).
	leftTurnDictionary add: (Association key: #SOUTH_EAST value: #EAST).
	leftTurnDictionary add: (Association key: #EAST value: #NORTH_EAST).
	leftTurnDictionary add: (Association key: #NORTH_EAST value: #NORTH).

	rightTurnDictionary := Dictionary new.
	rightTurnDictionary add: (Association key: #NORTH value: #NORTH_EAST).
	rightTurnDictionary add: (Association key: #NORTH_EAST value: #EAST).
	rightTurnDictionary add: (Association key: #EAST value: #SOUTH_EAST).
	rightTurnDictionary add: (Association key: #SOUTH_EAST value: #SOUTH).
	rightTurnDictionary add: (Association key: #SOUTH value: #SOUTH_WEST).
	rightTurnDictionary add: (Association key: #SOUTH_WEST value: #WEST).
	rightTurnDictionary add: (Association key: #WEST value: #NORTH_WEST).
	rightTurnDictionary add: (Association key: #NORTH_WEST value: #NORTH).
	
	reverseDirectionDictionary := Dictionary new.
	reverseDirectionDictionary add: (Association key: #NORTH value: #SOUTH).
	reverseDirectionDictionary add: (Association key: #NORTH_EAST value: #SOUTH_WEST).
	reverseDirectionDictionary add: (Association key: #EAST value: #WEST).
	reverseDirectionDictionary add: (Association key: #SOUTH_EAST value: #NORTH_WEST).
	reverseDirectionDictionary add: (Association key: #SOUTH value: #NORTH).
	reverseDirectionDictionary add: (Association key: #SOUTH_WEST value: #NORTH_EAST).
	reverseDirectionDictionary add: (Association key: #WEST value: #EAST).
	reverseDirectionDictionary add: (Association key: #NORTH_WEST value: #SOUTH_EAST).! !

!Environment class methodsFor: 'direction' stamp: 'VanceThornton 7/7/2012 00:39'!
reverseForDirection: direction
	^reverseDirectionDictionary at: direction! !

!Environment methodsFor: 'as yet unclassified' stamp: 'VanceThornton 7/3/2012 23:32'!
add: entity to: cell
	|status|
	status := EntityStatus new initWithCell: cell.
	entityStatusMap add: (Association key: entity value: status).
	cell add: entity.! !

!Environment methodsFor: 'accessing' stamp: 'VanceThornton 7/6/2012 22:41'!
"Responds with the cell in the given direction relative to the given cell"
cell: direction ofCell: cell
	cell == nil ifTrue: [self error: 'Nil cell'].
	direction = #NORTH ifTrue: [^self cellAtRow: (cell row - 1) column: (cell column)].
	direction = #NORTH_EAST ifTrue: [^self cellAtRow: (cell row - 1) column: (cell column + 1)].
	direction = #EAST ifTrue: [^self cellAtRow: (cell row) column: (cell column + 1)].
	direction = #SOUTH_EAST ifTrue: [^self cellAtRow: (cell row + 1) column: (cell column + 1)].
	direction = #SOUTH ifTrue: [^self cellAtRow: (cell row + 1) column: (cell column)].
	direction = #SOUTH_WEST ifTrue: [^self cellAtRow: (cell row + 1) column: (cell column - 1)].
	direction = #WEST ifTrue: [^self cellAtRow: (cell row) column: (cell column - 1)].
	direction = #NORTH_WEST ifTrue: [^self cellAtRow: (cell row - 1) column: (cell column - 1)].
	
	self error: 'Invalid direction'.! !

!Environment methodsFor: 'accessing' stamp: 'VanceThornton 7/6/2012 22:08'!
"Returns the cell at the specified row and column.  If the row and/or column is out of range then nil is returned"
cellAtRow: row column: column
	row < 1 ifTrue: [^nil].
	row > (self rowCount) ifTrue: [^nil].
	column < 1 ifTrue: [^nil].
	column > (self columnCount) ifTrue: [^nil].
	
	^cellMatrix at: row at: column
	! !

!Environment methodsFor: 'accessing' stamp: 'VanceThornton 7/6/2012 00:41'!
cellForEntity: entity
	^(entityStatusMap at: entity) cell.! !

!Environment methodsFor: 'accessing' stamp: 'VanceThornton 7/3/2012 18:40'!
columnCount
	^cellMatrix columnCount! !

!Environment methodsFor: 'accessing' stamp: 'VanceThornton 7/3/2012 23:15'!
entities
	^entityStatusMap keys! !

!Environment methodsFor: 'initialize-release' stamp: 'VanceThornton 7/4/2012 00:06'!
initWithRows: rowCount columns: columnCount
	cellMatrix := Matrix rows: rowCount columns: columnCount.
	cellMatrix indicesDo: [:row :column| cellMatrix at: row at: column put: (Cell new initWithRow: row column: column)].
	entityStatusMap := Dictionary new.! !

!Environment methodsFor: 'as yet unclassified' stamp: 'VanceThornton 7/6/2012 00:48'!
move: entity to: toCell
	|entityStatus fromCell|
	entityStatus := self statusForEntity: entity.
	fromCell := entityStatus cell.
	
	fromCell remove: entity.
	toCell add: entity.
	entityStatus cell: toCell.
! !

!Environment methodsFor: 'accessing' stamp: 'VanceThornton 7/3/2012 18:40'!
rowCount
	^cellMatrix rowCount! !

!Environment methodsFor: 'accessing' stamp: 'VanceThornton 7/3/2012 23:16'!
statusForEntity: entity
	^entityStatusMap at: entity! !

!Environment methodsFor: 'as yet unclassified' stamp: 'VanceThornton 7/6/2012 00:39'!
update
	(self entities) do: [:entity |
		|sensoryInput motorResponse|
		sensoryInput := (entity class) sensoryInputForEntity: entity inEnvironment: self.
		motorResponse := entity responseTo: sensoryInput.
		motorResponse performInEnvironment: self.
	].! !

Environment subclass: #SurfaceEnvironment
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AntSimulation'!

!SurfaceEnvironment class methodsFor: 'class initialization' stamp: 'VanceThornton 7/4/2012 00:14'!
generateRandomWithRows: rows 
	columns: columns 
	antCount: antCount
	
	|env|

	env := SurfaceEnvironment new initWithRows: rows columns: columns.
	
	antCount timesRepeat: [
		|ant cell row column|
		ant := Ant new.
		row := AntSimulation random nextInt: rows.
		column := AntSimulation random nextInt: columns.
		cell := env cellAtRow: row column: column.
		env add: ant to: cell.
	].
	^env! !

Object subclass: #MotorResponse
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AntSimulation'!

!MotorResponse methodsFor: 'as yet unclassified' stamp: 'VanceThornton 7/6/2012 00:35'!
performInEnvironment: env
	self subclassResponsibility! !

MotorResponse subclass: #MoveMotorResponse
	instanceVariableNames: 'entity direction'
	classVariableNames: 'reverseDirectionDictionary'
	poolDictionaries: ''
	category: 'AntSimulation'!

!MoveMotorResponse class methodsFor: 'class initialization' stamp: 'VanceThornton 7/6/2012 23:40'!
responseForEntity: entity moveDirection: direction
	^(MoveMotorResponse new entity: entity) direction: direction.! !

!MoveMotorResponse methodsFor: 'initialize-release' stamp: 'VanceThornton 7/6/2012 23:38'!
direction: aDirection
	direction := aDirection.! !

!MoveMotorResponse methodsFor: 'initialize-release' stamp: 'VanceThornton 7/6/2012 22:56'!
entity: anEntity
	entity := anEntity.! !

!MoveMotorResponse methodsFor: 'action' stamp: 'VanceThornton 7/7/2012 00:39'!
performInEnvironment: env
	|entityStatus currentCell moveDirection newCell|

	entityStatus := env statusForEntity: entity.

	moveDirection := entityStatus direction.
	(direction == #BACKWARD) ifTrue: [moveDirection := Environment reverseForDirection: moveDirection].
	
	currentCell := entityStatus cell.
	newCell := env cell: moveDirection ofCell: currentCell.
	
	newCell ~= nil ifTrue: [
		env move: entity to: newCell.
	].! !

MotorResponse subclass: #NoMotorResponse
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AntSimulation'!

!NoMotorResponse methodsFor: 'as yet unclassified' stamp: 'VanceThornton 7/6/2012 00:39'!
performInEnvironment: env! !

MotorResponse subclass: #TurnMotorResponse
	instanceVariableNames: 'entity direction'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AntSimulation'!

!TurnMotorResponse class methodsFor: 'class initialization' stamp: 'VanceThornton 7/6/2012 23:41'!
responseForEntity: entity turnDirection: direction
	^(TurnMotorResponse new entity: entity) direction: direction.! !

!TurnMotorResponse methodsFor: 'as yet unclassified' stamp: 'VanceThornton 7/6/2012 23:35'!
direction: aDirection
	direction := aDirection.! !

!TurnMotorResponse methodsFor: 'as yet unclassified' stamp: 'VanceThornton 7/6/2012 23:35'!
entity: anEntity
	entity := anEntity.! !

!TurnMotorResponse methodsFor: 'as yet unclassified' stamp: 'VanceThornton 7/7/2012 00:36'!
performInEnvironment: env
	|entityStatus currentDirection newDirection|

	entityStatus := env statusForEntity: entity.
	currentDirection := entityStatus direction.
	
	newDirection := Environment directionAfterTurning: direction from: currentDirection.
	entityStatus direction: newDirection.! !
Environment initialize!
